# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from .. import _legacy_response
from ..types import (
    ledger_account_balance_monitor_list_params,
    ledger_account_balance_monitor_create_params,
    ledger_account_balance_monitor_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_account_balance_monitor import LedgerAccountBalanceMonitor

__all__ = ["LedgerAccountBalanceMonitors", "AsyncLedgerAccountBalanceMonitors"]


class LedgerAccountBalanceMonitors(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LedgerAccountBalanceMonitorsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LedgerAccountBalanceMonitorsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgerAccountBalanceMonitorsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LedgerAccountBalanceMonitorsWithStreamingResponse(self)

    def create(
        self,
        *,
        alert_condition: ledger_account_balance_monitor_create_params.AlertCondition,
        ledger_account_id: str,
        description: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Create a ledger account balance monitor.

        Args:
          alert_condition: Describes the condition that must be satisfied for the monitor to be triggered.

          ledger_account_id: The ledger account associated with this balance monitor.

          description: An optional, free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/ledger_account_balance_monitors",
            body=maybe_transform(
                {
                    "alert_condition": alert_condition,
                    "ledger_account_id": ledger_account_id,
                    "description": description,
                    "metadata": metadata,
                },
                ledger_account_balance_monitor_create_params.LedgerAccountBalanceMonitorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerAccountBalanceMonitor:
        """
        Get details on a single ledger account balance monitor.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    def update(
        self,
        id: str,
        *,
        description: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Update a ledger account balance monitor.

        Args:
          description: An optional, free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            body=maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                },
                ledger_account_balance_monitor_update_params.LedgerAccountBalanceMonitorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        after_cursor: Optional[str] | Omit = omit,
        ledger_account_id: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[LedgerAccountBalanceMonitor]:
        """
        Get a list of ledger account balance monitors.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          ledger_account_id: Query the balance monitors for a single ledger account.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_account_balance_monitors",
            page=SyncPage[LedgerAccountBalanceMonitor],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "ledger_account_id": ledger_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                    },
                    ledger_account_balance_monitor_list_params.LedgerAccountBalanceMonitorListParams,
                ),
            ),
            model=LedgerAccountBalanceMonitor,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Delete a ledger account balance monitor.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )


class AsyncLedgerAccountBalanceMonitors(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLedgerAccountBalanceMonitorsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgerAccountBalanceMonitorsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgerAccountBalanceMonitorsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLedgerAccountBalanceMonitorsWithStreamingResponse(self)

    async def create(
        self,
        *,
        alert_condition: ledger_account_balance_monitor_create_params.AlertCondition,
        ledger_account_id: str,
        description: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Create a ledger account balance monitor.

        Args:
          alert_condition: Describes the condition that must be satisfied for the monitor to be triggered.

          ledger_account_id: The ledger account associated with this balance monitor.

          description: An optional, free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/ledger_account_balance_monitors",
            body=await async_maybe_transform(
                {
                    "alert_condition": alert_condition,
                    "ledger_account_id": ledger_account_id,
                    "description": description,
                    "metadata": metadata,
                },
                ledger_account_balance_monitor_create_params.LedgerAccountBalanceMonitorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerAccountBalanceMonitor:
        """
        Get details on a single ledger account balance monitor.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    async def update(
        self,
        id: str,
        *,
        description: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Update a ledger account balance monitor.

        Args:
          description: An optional, free-form description for internal use.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                },
                ledger_account_balance_monitor_update_params.LedgerAccountBalanceMonitorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )

    def list(
        self,
        *,
        id: SequenceNotStr[str] | Omit = omit,
        after_cursor: Optional[str] | Omit = omit,
        ledger_account_id: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LedgerAccountBalanceMonitor, AsyncPage[LedgerAccountBalanceMonitor]]:
        """
        Get a list of ledger account balance monitors.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          ledger_account_id: Query the balance monitors for a single ledger account.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_account_balance_monitors",
            page=AsyncPage[LedgerAccountBalanceMonitor],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "ledger_account_id": ledger_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                    },
                    ledger_account_balance_monitor_list_params.LedgerAccountBalanceMonitorListParams,
                ),
            ),
            model=LedgerAccountBalanceMonitor,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> LedgerAccountBalanceMonitor:
        """
        Delete a ledger account balance monitor.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/api/ledger_account_balance_monitors/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountBalanceMonitor,
        )


class LedgerAccountBalanceMonitorsWithRawResponse:
    def __init__(self, ledger_account_balance_monitors: LedgerAccountBalanceMonitors) -> None:
        self._ledger_account_balance_monitors = ledger_account_balance_monitors

        self.create = _legacy_response.to_raw_response_wrapper(
            ledger_account_balance_monitors.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            ledger_account_balance_monitors.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            ledger_account_balance_monitors.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            ledger_account_balance_monitors.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            ledger_account_balance_monitors.delete,
        )


class AsyncLedgerAccountBalanceMonitorsWithRawResponse:
    def __init__(self, ledger_account_balance_monitors: AsyncLedgerAccountBalanceMonitors) -> None:
        self._ledger_account_balance_monitors = ledger_account_balance_monitors

        self.create = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_balance_monitors.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_balance_monitors.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_balance_monitors.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_balance_monitors.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_balance_monitors.delete,
        )


class LedgerAccountBalanceMonitorsWithStreamingResponse:
    def __init__(self, ledger_account_balance_monitors: LedgerAccountBalanceMonitors) -> None:
        self._ledger_account_balance_monitors = ledger_account_balance_monitors

        self.create = to_streamed_response_wrapper(
            ledger_account_balance_monitors.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledger_account_balance_monitors.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ledger_account_balance_monitors.update,
        )
        self.list = to_streamed_response_wrapper(
            ledger_account_balance_monitors.list,
        )
        self.delete = to_streamed_response_wrapper(
            ledger_account_balance_monitors.delete,
        )


class AsyncLedgerAccountBalanceMonitorsWithStreamingResponse:
    def __init__(self, ledger_account_balance_monitors: AsyncLedgerAccountBalanceMonitors) -> None:
        self._ledger_account_balance_monitors = ledger_account_balance_monitors

        self.create = async_to_streamed_response_wrapper(
            ledger_account_balance_monitors.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledger_account_balance_monitors.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ledger_account_balance_monitors.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ledger_account_balance_monitors.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ledger_account_balance_monitors.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .account_detail import AccountDetail
from .routing_detail import RoutingDetail
from .shared.address import Address
from .external_account_type import ExternalAccountType
from .shared.contact_detail import ContactDetail

__all__ = ["Counterparty", "Accounts", "Account"]


class Account(BaseModel):
    id: Optional[str] = None

    account_details: Optional[List[AccountDetail]] = None

    account_type: Optional[ExternalAccountType] = None
    """Can be `checking`, `savings` or `other`."""

    contact_details: Optional[List[ContactDetail]] = None

    created_at: Optional[datetime] = None

    discarded_at: Optional[datetime] = None

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    ledger_account_id: Optional[str] = None
    """
    If the external account links to a ledger account in Modern Treasury, the id of
    the ledger account will be populated here.
    """

    live_mode: Optional[bool] = None
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Optional[Dict[str, str]] = None
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str] = None
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    object: Optional[str] = None

    party_address: Optional[Address] = None
    """The address associated with the owner or `null`."""

    party_name: Optional[str] = None
    """The legal name of the entity which owns the account."""

    party_type: Optional[Literal["business", "individual"]] = None
    """Either `individual` or `business`."""

    routing_details: Optional[List[RoutingDetail]] = None

    updated_at: Optional[datetime] = None

    verification_source: Optional[Literal["ach_prenote", "microdeposits", "plaid"]] = None

    verification_status: Optional[Literal["pending_verification", "unverified", "verified"]] = None


Accounts = Account
"""This type is deprecated and will be removed in a future release.

Please use Account instead.
"""


class Counterparty(BaseModel):
    id: str

    accounts: List[Account]
    """The accounts for this counterparty."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    email: Optional[str] = None
    """The counterparty's email."""

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    legal_entity_id: Optional[str] = None
    """The id of the legal entity."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str] = None
    """A human friendly name for this counterparty."""

    object: str

    send_remittance_advice: bool
    """
    Send an email to the counterparty whenever an associated payment order is sent
    to the bank.
    """

    updated_at: datetime

    verification_status: str
    """The verification status of the counterparty."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import RoutingNumberLookupRequest

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestValidations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_validate_routing_number(self, client: ModernTreasury) -> None:
        validation = client.validations.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        )
        assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

    @parametrize
    def test_raw_response_validate_routing_number(self, client: ModernTreasury) -> None:
        response = client.validations.with_raw_response.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        validation = response.parse()
        assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

    @parametrize
    def test_streaming_response_validate_routing_number(self, client: ModernTreasury) -> None:
        with client.validations.with_streaming_response.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            validation = response.parse()
            assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncValidations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_validate_routing_number(self, async_client: AsyncModernTreasury) -> None:
        validation = await async_client.validations.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        )
        assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

    @parametrize
    async def test_raw_response_validate_routing_number(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.validations.with_raw_response.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        validation = response.parse()
        assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

    @parametrize
    async def test_streaming_response_validate_routing_number(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.validations.with_streaming_response.validate_routing_number(
            routing_number="routing_number",
            routing_number_type="aba",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            validation = await response.parse()
            assert_matches_type(RoutingNumberLookupRequest, validation, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import builtins
from typing import List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.currency import Currency

__all__ = ["ReturnObject", "Corrections", "ReferenceNumbers", "ReferenceNumber"]


class Corrections(BaseModel):
    """Only relevant for ACH NOC returns.

    This is an object containing all of the new and corrected information provided by the bank that was previously incorrect on the original outgoing payment.
    """

    account_number: Optional[str] = None
    """
    The updated account number that should replace the one originally used on the
    outgoing payment.
    """

    company_id: Optional[str] = None
    """
    The updated company ID that should replace the one originally used on the
    outgoing payment.
    """

    company_name: Optional[str] = None
    """
    The updated company name that should replace the one originally used on the
    outgoing payment.
    """

    individual_identification_number: Optional[str] = None
    """
    The updated individual identification number that should replace the one
    originally used on the outgoing payment.
    """

    routing_number: Optional[str] = None
    """
    The updated routing number that should replace the one originally used on the
    outgoing payment.
    """

    transaction_code: Optional[str] = None
    """
    The updated account type code that should replace the one originally used on the
    outgoing payment.
    """


class ReferenceNumber(BaseModel):
    id: str

    created_at: datetime

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    reference_number: str
    """The vendor reference number."""

    reference_number_type: Literal[
        "ach_original_trace_number",
        "ach_trace_number",
        "bankprov_payment_activity_date",
        "bankprov_payment_id",
        "blockchain_transaction_hash",
        "bnk_dev_prenotification_id",
        "bnk_dev_transfer_id",
        "bny_mellon_transaction_reference_number",
        "bofa_end_to_end_id",
        "bofa_transaction_id",
        "brale_transfer_id",
        "check_number",
        "chips_uid",
        "citibank_reference_number",
        "citibank_worldlink_clearing_system_reference_number",
        "column_fx_quote_id",
        "column_reversal_pair_transfer_id",
        "column_transfer_id",
        "cross_river_card_trace_number",
        "cross_river_core_transaction_id",
        "cross_river_fed_batch_id",
        "cross_river_payment_id",
        "cross_river_retrieval_reference_id",
        "cross_river_service_message",
        "cross_river_transaction_id",
        "currencycloud_conversion_id",
        "currencycloud_payment_id",
        "dc_bank_transaction_id",
        "eft_trace_number",
        "evolve_core_batch",
        "evolve_core_file_key",
        "evolve_core_seq",
        "evolve_transaction_id",
        "fake_vendor_payment_id",
        "fedwire_imad",
        "fedwire_omad",
        "first_republic_internal_id",
        "goldman_sachs_collection_request_id",
        "goldman_sachs_end_to_end_id",
        "goldman_sachs_payment_request_id",
        "goldman_sachs_request_id",
        "goldman_sachs_unique_payment_id",
        "interac_message_id",
        "jpmc_ccn",
        "jpmc_clearing_system_reference",
        "jpmc_customer_reference_id",
        "jpmc_end_to_end_id",
        "jpmc_firm_root_id",
        "jpmc_fx_trn_id",
        "jpmc_p3_id",
        "jpmc_payment_batch_id",
        "jpmc_payment_information_id",
        "jpmc_payment_returned_datetime",
        "jpmc_transaction_reference_number",
        "lob_check_id",
        "other",
        "partial_swift_mir",
        "paxos_fiat_withdrawal_id",
        "paxos_group_id",
        "paxos_orchestration_id",
        "paxos_transfer_id",
        "pnc_clearing_reference",
        "pnc_instruction_id",
        "pnc_multipayment_id",
        "pnc_payment_trace_id",
        "pnc_request_for_payment_id",
        "pnc_transaction_reference_number",
        "rbc_wire_reference_id",
        "rtp_instruction_id",
        "signet_api_reference_id",
        "signet_confirmation_id",
        "signet_request_id",
        "silvergate_payment_id",
        "svb_end_to_end_id",
        "svb_payment_id",
        "swift_mir",
        "swift_uetr",
        "umb_product_partner_account_number",
        "usbank_payment_application_reference_id",
        "usbank_payment_id",
        "usbank_pending_rtp_payment_id",
        "usbank_posted_rtp_payment_id",
        "wells_fargo_end_to_end_id",
        "wells_fargo_payment_id",
        "wells_fargo_trace_number",
        "wells_fargo_uetr",
        "western_alliance_payment_id",
        "western_alliance_transaction_id",
        "western_alliance_wire_confirmation_number",
    ]
    """The type of the reference number. Referring to the vendor payment id."""

    updated_at: datetime


ReferenceNumbers = ReferenceNumber
"""This type is deprecated and will be removed in a future release.

Please use ReferenceNumber instead.
"""


class ReturnObject(BaseModel):
    id: str

    amount: int
    """Value in specified currency's smallest unit.

    e.g. $10 would be represented as 1000.
    """

    code: Optional[
        Literal[
            "901",
            "902",
            "903",
            "904",
            "905",
            "907",
            "908",
            "909",
            "910",
            "911",
            "912",
            "914",
            "C01",
            "C02",
            "C03",
            "C05",
            "C06",
            "C07",
            "C08",
            "C09",
            "C13",
            "C14",
            "R01",
            "R02",
            "R03",
            "R04",
            "R05",
            "R06",
            "R07",
            "R08",
            "R09",
            "R10",
            "R11",
            "R12",
            "R13",
            "R14",
            "R15",
            "R16",
            "R17",
            "R18",
            "R19",
            "R20",
            "R21",
            "R22",
            "R23",
            "R24",
            "R25",
            "R26",
            "R27",
            "R28",
            "R29",
            "R30",
            "R31",
            "R32",
            "R33",
            "R34",
            "R35",
            "R36",
            "R37",
            "R38",
            "R39",
            "R40",
            "R41",
            "R42",
            "R43",
            "R44",
            "R45",
            "R46",
            "R47",
            "R50",
            "R51",
            "R52",
            "R53",
            "R61",
            "R62",
            "R67",
            "R68",
            "R69",
            "R70",
            "R71",
            "R72",
            "R73",
            "R74",
            "R75",
            "R76",
            "R77",
            "R80",
            "R81",
            "R82",
            "R83",
            "R84",
            "R85",
            "currencycloud",
        ]
    ] = None
    """The return code. For ACH returns, this is the required ACH return code."""

    corrections: Optional[Corrections] = None
    """Only relevant for ACH NOC returns.

    This is an object containing all of the new and corrected information provided
    by the bank that was previously incorrect on the original outgoing payment.
    """

    created_at: datetime

    currency: Currency
    """Currency that this transaction is denominated in."""

    current_return: Optional["ReturnObject"] = None
    """
    If the return's status is `returned`, this will include the return object's data
    that is returning this return.
    """

    date_of_death: Optional[date] = None
    """
    If the return code is `R14` or `R15` this is the date the deceased counterparty
    passed away.
    """

    discarded_at: Optional[datetime] = None

    failure_reason: Optional[str] = None
    """
    If an originating return failed to be processed by the bank, a description of
    the failure reason will be available.
    """

    internal_account_id: Optional[str] = None
    """The ID of the relevant Internal Account."""

    ledger_transaction_id: Optional[str] = None
    """The ID of the ledger transaction linked to the return."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    reason: Optional[str] = None
    """
    Often the bank will provide an explanation for the return, which is a short
    human readable string.
    """

    reconciliation_status: Literal["unreconciled", "tentatively_reconciled", "reconciled"]
    """One of `unreconciled`, `tentatively_reconciled` or `reconciled`."""

    reference_numbers: List[ReferenceNumber]
    """An array of Payment Reference objects."""

    returnable_id: Optional[str] = None
    """The ID of the object being returned or `null`."""

    returnable_type: Optional[Literal["incoming_payment_detail", "payment_order", "return", "reversal"]] = None
    """The type of object being returned or `null`."""

    role: Literal["originating", "receiving"]
    """The role of the return, can be `originating` or `receiving`."""

    status: Literal["cancelled", "completed", "failed", "pending", "processing", "returned", "sent"]
    """The current status of the return."""

    transaction_id: Optional[str] = None
    """The ID of the relevant Transaction or `null`."""

    transaction_line_item_id: Optional[str] = None
    """The ID of the relevant Transaction Line Item or `null`."""

    type: Literal[
        "ach",
        "ach_noc",
        "au_becs",
        "bacs",
        "book",
        "check",
        "cross_border",
        "eft",
        "gb_fps",
        "interac",
        "manual",
        "sepa",
        "wire",
        "zengin",
    ]
    """The type of return.

    Can be one of: `ach`, `ach_noc`, `au_becs`, `bacs`, `eft`, `interac`, `manual`,
    `paper_item`, `wire`.
    """

    updated_at: datetime

    additional_information: Optional[str] = None
    """Some returns may include additional information from the bank.

    In these cases, this string will be present.
    """

    data: Optional[builtins.object] = None
    """The raw data from the return file that we get from the bank."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["AddressRequest"]


class AddressRequest(TypedDict, total=False):
    country: Optional[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Optional[str]

    line2: Optional[str]

    locality: Optional[str]
    """Locality or City."""

    postal_code: Optional[str]
    """The postal code of the address."""

    region: Optional[str]
    """Region or State."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

from ..shared_params.ledger_transaction_create_request import LedgerTransactionCreateRequest

__all__ = ["ReversalCreateParams"]


class ReversalCreateParams(TypedDict, total=False):
    reason: Required[
        Literal[
            "duplicate",
            "incorrect_amount",
            "incorrect_receiving_account",
            "date_earlier_than_intended",
            "date_later_than_intended",
        ]
    ]
    """The reason for the reversal.

    Must be one of `duplicate`, `incorrect_amount`, `incorrect_receiving_account`,
    `date_earlier_than_intended`, `date_later_than_intended`.
    """

    ledger_transaction: LedgerTransactionCreateRequest
    """Specifies a ledger transaction object that will be created with the reversal.

    If the ledger transaction cannot be created, then the reversal creation will
    fail. The resulting ledger transaction will mirror the status of the reversal.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, TypedDict

from .external_account_type import ExternalAccountType
from .shared_params.address_request import AddressRequest

__all__ = ["ExternalAccountUpdateParams"]


class ExternalAccountUpdateParams(TypedDict, total=False):
    account_type: ExternalAccountType
    """Can be `checking`, `savings` or `other`."""

    counterparty_id: Optional[str]

    metadata: Dict[str, str]
    """Additional data in the form of key-value pairs.

    Pairs can be removed by passing an empty string or `null` as the value.
    """

    name: Optional[str]
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    party_address: AddressRequest

    party_name: str
    """
    If this value isn't provided, it will be inherited from the counterparty's name.
    """

    party_type: Optional[Literal["business", "individual"]]
    """Either `individual` or `business`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.currency import Currency

__all__ = ["ForeignExchangeQuoteCreateParams"]


class ForeignExchangeQuoteCreateParams(TypedDict, total=False):
    internal_account_id: Required[str]
    """The ID for the `InternalAccount` this quote is associated with."""

    target_currency: Required[Currency]
    """Currency to convert the `base_currency` to, often called the "buy" currency."""

    base_amount: int
    """
    Amount in the lowest denomination of the `base_currency` to convert, often
    called the "sell" amount.
    """

    base_currency: Currency
    """Currency to convert, often called the "sell" currency."""

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The timestamp until when the quoted rate is valid."""

    target_amount: int
    """
    Amount in the lowest denomination of the `target_currency`, often called the
    "buy" amount.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ConnectionListParams"]


class ConnectionListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    entity: str
    """A string code representing the vendor (i.e. bank)."""

    per_page: int

    vendor_customer_id: str
    """An identifier assigned by the vendor to your organization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date, datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ForeignExchangeQuoteListParams"]


class ForeignExchangeQuoteListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    base_currency: str
    """Currency to convert, often called the "sell" currency."""

    effective_at_end: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """An inclusive upper bound for searching effective_at"""

    effective_at_start: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """An inclusive lower bound for searching effective_at"""

    expires_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The timestamp until which the quote must be booked by."""

    internal_account_id: str
    """The ID for the `InternalAccount` this quote is associated with."""

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    target_currency: str
    """Currency to convert the `base_currency` to, often called the "buy" currency."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ThirdPartyVerification"]


class ThirdPartyVerification(BaseModel):
    outcome: Literal["passed", "failed"]
    """The outcome of the verification. One of `passed` or `failed`."""

    vendor: Literal["persona", "middesk", "alloy", "sumsub", "veriff"]
    """The vendor that performed the verification, e.g. `persona`."""

    vendor_verification_id: str
    """The identification of the third party verification in `vendor`'s system."""

    verification_category: Literal["legal_name", "date_of_birth", "address", "government_id_number", "adverse_media"]
    """The category of verification performed."""

    verification_method: str
    """The method used to perform the verification."""

    verification_time: datetime
    """The timestamp when the verification was performed."""

    comment: Optional[str] = None
    """An optional comment about the verification."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.currency import Currency
from .expected_payment_type import ExpectedPaymentType
from .reconciliation_rule_param import ReconciliationRuleParam
from .shared_params.ledger_transaction_create_request import LedgerTransactionCreateRequest

__all__ = ["ExpectedPaymentCreateParams", "LineItems", "LineItem"]


class ExpectedPaymentCreateParams(TypedDict, total=False):
    amount_lower_bound: Optional[int]
    """The lowest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_reconciled: Optional[int]
    """The amount reconciled for this expected payment.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_reconciled_direction: Optional[Literal["credit", "debit"]]
    """One of credit or debit.

    Indicates whether amount_reconciled is a credit or debit amount.
    """

    amount_unreconciled: Optional[int]
    """The amount that remains unreconciled for this expected payment.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_unreconciled_direction: Optional[Literal["credit", "debit"]]
    """One of credit or debit.

    Indicates whether amount_unreconciled is a credit or debit amount.
    """

    amount_upper_bound: Optional[int]
    """The highest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    counterparty_id: Optional[str]
    """The ID of the counterparty you expect for this payment."""

    currency: Optional[Currency]
    """Must conform to ISO 4217. Defaults to the currency of the internal account."""

    date_lower_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The earliest date the payment may come in. Format: yyyy-mm-dd"""

    date_upper_bound: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """The latest date the payment may come in. Format: yyyy-mm-dd"""

    description: Optional[str]
    """An optional description for internal use."""

    direction: Optional[Literal["credit", "debit"]]
    """One of credit or debit.

    When you are receiving money, use credit. When you are being charged, use debit.
    """

    external_id: Optional[str]
    """An optional user-defined 180 character unique identifier."""

    internal_account_id: Optional[str]
    """The ID of the Internal Account for the expected payment."""

    ledger_transaction: LedgerTransactionCreateRequest
    """
    Specifies a ledger transaction object that will be created with the expected
    payment. If the ledger transaction cannot be created, then the expected payment
    creation will fail. The resulting ledger transaction will mirror the status of
    the expected payment.
    """

    ledger_transaction_id: str
    """Either ledger_transaction or ledger_transaction_id can be provided.

    Only a pending ledger transaction can be attached upon expected payment
    creation. Once the expected payment is created, the status of the ledger
    transaction tracks the expected payment automatically.
    """

    line_items: Iterable[LineItem]

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    reconciliation_filters: Optional[object]
    """The reconciliation filters you have for this payment."""

    reconciliation_groups: Optional[object]
    """The reconciliation groups you have for this payment."""

    reconciliation_rule_variables: Optional[Iterable[ReconciliationRuleParam]]
    """An array of reconciliation rule variables for this payment."""

    remittance_information: Optional[str]
    """For `ach`, this field will be passed through on an addenda record.

    For `wire` payments the field will be passed through as the "Originator to
    Beneficiary Information", also known as OBI or Fedwire tag 6000.
    """

    statement_descriptor: Optional[str]
    """The statement description you expect to see on the transaction.

    For ACH payments, this will be the full line item passed from the bank. For wire
    payments, this will be the OBI field on the wire. For check payments, this will
    be the memo field.
    """

    type: Optional[ExpectedPaymentType]
    """
    One of: ach, au_becs, bacs, book, check, eft, interac, provxchange, rtp, sen,
    sepa, signet, wire.
    """


class LineItem(TypedDict, total=False):
    amount: Required[int]
    """Value in specified currency's smallest unit.

    e.g. $10 would be represented as 1000.
    """

    accounting_category_id: Optional[str]
    """The ID of one of your accounting categories.

    Note that these will only be accessible if your accounting system has been
    connected.
    """

    description: Optional[str]
    """A free-form description of the line item."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """


LineItems = LineItem
"""This type is deprecated and will be removed in a future release.

Please use LineItem instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .currency import Currency
from ..._models import BaseModel

__all__ = ["ForeignExchangeRate"]


class ForeignExchangeRate(BaseModel):
    base_amount: int
    """
    Amount in the lowest denomination of the `base_currency` to convert, often
    called the "sell" amount.
    """

    base_currency: Currency
    """Currency to convert, often called the "sell" currency."""

    exponent: int
    """The exponent component of the rate.

    The decimal is calculated as `value` / (10 ^ `exponent`).
    """

    rate_string: str
    """A string representation of the rate."""

    target_amount: int
    """
    Amount in the lowest denomination of the `target_currency`, often called the
    "buy" amount.
    """

    target_currency: Currency
    """Currency to convert the `base_currency` to, often called the "buy" currency."""

    value: int
    """The whole number component of the rate.

    The decimal is calculated as `value` / (10 ^ `exponent`).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import TYPE_CHECKING, Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .connection import Connection
from .account_detail import AccountDetail
from .routing_detail import RoutingDetail
from .shared.address import Address
from .shared.currency import Currency
from .shared.transaction_direction import TransactionDirection

__all__ = ["InternalAccount", "AccountCapabilities", "AccountCapability"]


class AccountCapability(BaseModel):
    id: str

    created_at: datetime

    direction: TransactionDirection
    """One of `debit` or `credit`.

    Indicates the direction of money movement this capability is responsible for.
    """

    discarded_at: Optional[datetime] = None

    identifier: Optional[str] = None
    """
    A unique reference assigned by your bank for tracking and recognizing payment
    files. It is important this is formatted exactly how the bank assigned it.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    payment_type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "gb_fps",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "stablecoin",
        "wire",
        "zengin",
    ]
    """
    Indicates the the type of payment this capability is responsible for
    originating.
    """

    updated_at: datetime

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, builtins.object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> builtins.object: ...
    else:
        __pydantic_extra__: Dict[str, builtins.object]


AccountCapabilities = AccountCapability
"""This type is deprecated and will be removed in a future release.

Please use AccountCapability instead.
"""


class InternalAccount(BaseModel):
    id: str

    account_capabilities: List[AccountCapability]
    """
    An array of AccountCapability objects that list the originating abilities of the
    internal account and any relevant information for them.
    """

    account_details: List[AccountDetail]
    """An array of account detail objects."""

    account_type: Optional[
        Literal[
            "base_wallet",
            "cash",
            "checking",
            "crypto_wallet",
            "ethereum_wallet",
            "general_ledger",
            "loan",
            "non_resident",
            "other",
            "overdraft",
            "polygon_wallet",
            "savings",
            "solana_wallet",
        ]
    ] = None
    """Can be checking, savings or other."""

    connection: Connection
    """Specifies which financial institution the accounts belong to."""

    contra_ledger_account_id: Optional[str] = None
    """
    If the internal account links to a contra ledger account in Modern Treasury, the
    id of the contra ledger account will be populated here.
    """

    counterparty_id: Optional[str] = None
    """The Counterparty associated to this account."""

    created_at: datetime

    currency: Currency
    """The currency of the account."""

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    ledger_account_id: Optional[str] = None
    """
    If the internal account links to a ledger account in Modern Treasury, the id of
    the ledger account will be populated here.
    """

    legal_entity_id: Optional[str] = None
    """The Legal Entity associated to this account."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str] = None
    """A nickname for the account."""

    object: str

    parent_account_id: Optional[str] = None
    """The parent InternalAccount of this account."""

    party_address: Optional[Address] = None
    """The address associated with the owner or null."""

    party_name: str
    """The legal name of the entity which owns the account."""

    party_type: Optional[Literal["business", "individual"]] = None
    """Either individual or business."""

    routing_details: List[RoutingDetail]
    """An array of routing detail objects."""

    status: Optional[Literal["active", "closed", "pending_activation", "pending_closure", "suspended"]] = None
    """The internal account status."""

    updated_at: datetime

    vendor_id: Optional[str] = None
    """The vendor ID associated with this account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["LedgerAccountRetrieveParams", "Balances"]


class LedgerAccountRetrieveParams(TypedDict, total=False):
    balances: Balances
    """
    Use `balances[effective_at_lower_bound]` and
    `balances[effective_at_upper_bound]` to get the balances change between the two
    timestamps. The lower bound is inclusive while the upper bound is exclusive of
    the provided timestamps. If no value is supplied the balances will be retrieved
    not including that bound. Use `balances[as_of_lock_version]` to retrieve a
    balance as of a specific Ledger Account `lock_version`.
    """


class Balances(TypedDict, total=False):
    """
    Use `balances[effective_at_lower_bound]` and `balances[effective_at_upper_bound]` to get the balances change between the two timestamps. The lower bound is inclusive while the upper bound is exclusive of the provided timestamps. If no value is supplied the balances will be retrieved not including that bound. Use `balances[as_of_lock_version]` to retrieve a balance as of a specific Ledger Account `lock_version`.
    """

    as_of_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]

    as_of_lock_version: int

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    effective_at_lower_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    effective_at_upper_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["InternalAccountUpdateParams"]


class InternalAccountUpdateParams(TypedDict, total=False):
    contra_ledger_account_id: str
    """The Contra Ledger Account associated to this account."""

    counterparty_id: str
    """The Counterparty associated to this account."""

    external_id: Optional[str]
    """An optional user-defined 180 character unique identifier."""

    ledger_account_id: str
    """The Ledger Account associated to this account."""

    metadata: Dict[str, str]
    """Additional data in the form of key-value pairs.

    Pairs can be removed by passing an empty string or `null` as the value.
    """

    name: str
    """The nickname for the internal account."""

    parent_account_id: str
    """The parent internal account for this account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel
from .external_account import ExternalAccount

__all__ = ["ExternalAccountVerifyResponse", "ExternalAccountVerificationAttempt"]


class ExternalAccountVerificationAttempt(BaseModel):
    id: str

    created_at: datetime

    external_account_id: str
    """The ID of the external account."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    originating_account_id: str
    """The ID of the internal account where the micro-deposits originate from."""

    payment_type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "gb_fps",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "stablecoin",
        "wire",
        "zengin",
    ]
    """The type of payment that can be made to this account.

    Can be `ach`, `eft`, or `rtp`.
    """

    priority: Optional[Literal["high", "normal"]] = None
    """The priority of the payment. Can be `normal` or `high`."""

    status: Literal["cancelled", "failed", "pending_verification", "verified"]
    """The status of the verification attempt.

    Can be `pending_verification`, `verified`, `failed`, or `cancelled`.
    """

    updated_at: datetime


ExternalAccountVerifyResponse: TypeAlias = Union[ExternalAccount, ExternalAccountVerificationAttempt]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.transaction_direction import TransactionDirection

__all__ = ["ExpectedPaymentListParams"]


class ExpectedPaymentListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    counterparty_id: str
    """Specify counterparty_id to see expected_payments for a specific account."""

    created_at_lower_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return expected payments created after some datetime"""

    created_at_upper_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return expected payments created before some datetime"""

    direction: TransactionDirection
    """One of credit, debit"""

    external_id: str

    internal_account_id: str
    """Specify internal_account_id to see expected_payments for a specific account."""

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    status: Literal["archived", "partially_reconciled", "reconciled", "unreconciled"]
    """One of unreconciled, reconciled, or archived."""

    type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "gb_fps",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "stablecoin",
        "wire",
        "zengin",
    ]
    """
    One of: ach, au_becs, bacs, book, check, eft, interac, provxchange, rtp,sen,
    sepa, signet, wire
    """

    updated_at_lower_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return expected payments updated after some datetime"""

    updated_at_upper_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Used to return expected payments updated before some datetime"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["LegalEntityIndustryClassification"]


class LegalEntityIndustryClassification(BaseModel):
    id: str

    classification_codes: List[str]
    """The industry classification codes for the legal entity."""

    classification_type: Literal[
        "anzsic",
        "bics",
        "gics",
        "hsics",
        "icb",
        "isic",
        "mgecs",
        "nace",
        "naics",
        "rbics",
        "sic",
        "sni",
        "trbc",
        "uksic",
        "unspsc",
    ]
    """The classification system of the classification codes."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: datetime
